function [He,Chh]=calc_homography_4_points(Xe,Ye,Cxxe,Ht)
%%Homography from 4 point pairs
%
% Xh    2x4   matrix of homogeneous coordiantes of 4 points (fixed)
% Yh    2*4   matrix of homogeneous coordiantes of 4 points (noisy)
% Cxx   8x8 covariance matrix of vec(Ym), may be full
% Ht    3x3   true homography (for testing), Frobenius norm = 1
%
% He    3x3 matrix, estimated homography
% Chh   9x9 covariance matrix of vec(He)
%
% wf 3/2024

%% set variable for test
no_test = (nargin == 3);

% homogeneous coordinates and CovM
Xh  = [Xe(1:2,:);ones(1,4)];
Yh  = [Ye(1:2,:);ones(1,4)];

Cxx=zeros(12);
for i=1:4
    for j = 1:4
        Cxx(3*i-2:3*i-1,3*j-2:3*j-1)=Cxxe(2*i-1:2*i,2*j-1:2*j);
    end
end

%% condition points and covariance matrix

% mean, stdv., and conditioning matrices
cx  = mean(Xe')';
sx  = sqrt(var(Xe(1,:))+var(Xe(2,:)));
Mx  = [eye(2)/sx -cx/sx;0 0 1];

cy  = mean(Ye')';
sy  = sqrt(var(Ye(1,:))+var(Ye(2,:)));
My  = [eye(2)/sy -cy/sy;0 0 1];
% Mx=eye(3);
% My=eye(3);

% conditioning
Xhc = Mx*Xh;
Yhc = My*Yh;
Xec = Xhc(1:2,:)./([1;1]*Xhc(3,:));
Yec = Yhc(1:2,:)./([1;1]*Yhc(3,:));


% condition true homography and CovM
if no_test == 0
Htc  = (My)*Ht*inv(Mx);
end

Mc = kron(eye(4),My);  
Cxxc= Mc*Cxx*Mc';
%% build design matrix and conditioned CovM
A0=zeros(8,9);
for i = 1:4
    xt=Xhc(:,i)';
    yt=Yhc(:,i)';
    z3=zeros(1,3);
    S = [eye(2), [0;0]]*calc_S(Yhc(:,i));
    A0(2*i-1:2*i,:)=...
        [xt(1)*S xt(2)*S xt(3)*S];
end

%% solve, enforce positive determinant

hec    = null(A0);
Hec    = reshape(hec,3,3);
signum = sign(det(Hec));
hec    = hec*signum;
Hec    = Hec*signum;

%% derive CovM
% Jacobian dk wrt parameters
A = [A0;hec'];
% jacobian dk wrt observations
B=zeros(9,12);
for i=1:4
    B(2*i-1:2*i,3*i-2:3*i)=[eye(2) zeros(2,1)]*calc_S(Hec*Xhc(:,i));
end
% Jacobian for variance implicit propagation
J=A\B;
Chhc = J*Cxxc*J';


%% compare He with Ht (in conditioned frames)
if no_test==0
dH = (Htc-Hec)
Xcond = dH(:)'*pinv(Chhc)*dH(:);
disp(['test conditioned X = ', num2str(Xcond),...
    ' interval = [',num2str(chi2inv(0.005,8)),',',num2str(chi2inv(0.995,8)),']'])
end

%% un-condition H and Chh
He  = inv(My)*Hec*Mx;
He  = He / norm(He(:))*sign(det(He))
Mc  = kron(Mx',inv(My));
Chh = Mc*Chhc*Mc';

%% check hc -> h, compare He with Ht (in un-conditioned frames)
if no_test==0
    
difference_hs=(Mc*hec(:)-Ht(:))'

dH = (Ht-He);
X = dH(:)'*pinv(Chh)*dH(:);
disp(['test un-conditioned X = ', num2str(X),...
    ' interval = [',num2str(chi2inv(0.005,8)),',',num2str(chi2inv(0.995,8)),']'])

end


end

